@@ -4,7 +4,7 @@ use strict;
 use warnings FATAL => 'all';
 
 # Initialize our version
-our $VERSION = (qw($Revision: 1.06 $))[1];
+our $VERSION = (qw($Revision: 1.07 $))[1];
 
 # Use Error.pm's try/catch semantics
 use Error qw( :try );
@@ -13,7 +13,7 @@ use Error qw( :try );
 use POE::Filter::Reference;
 
 # We run the actual DB connection here
-use DBI;
+use DBI qw(:sql_types);
 
 sub new {
 	my ($class, $opts) = @_;
@@ -33,6 +33,9 @@ sub main {
 	select(STDOUT); $|++;
 	select(STDERR);	$|++;
 
+    $SIG{__WARN__} = 'DEFAULT';
+    $SIG{__DIE__} = 'DEFAULT';
+
 	my $self;
 	# check for alternate fork
 	if ($_[0] == 1) {
@@ -478,9 +481,10 @@ sub db_single {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -590,7 +594,8 @@ sub db_insert {
 			} else {
 				# Execute the query
 				try {
-					$rows_affected += $sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$rows_affected += $sth->execute();
 				} catch Error with {
 					if (defined($sth->errstr)) {
 						die $sth->errstr;
@@ -718,9 +723,10 @@ sub db_do {
 			} else {
 				# Execute the query
 				try {
-					$rows_affected = $sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$rows_affected += $sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -791,9 +797,10 @@ sub db_arrayhash {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -905,7 +912,8 @@ sub db_hashhash {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
 					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
@@ -1042,9 +1050,10 @@ sub db_hasharray {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -1170,9 +1179,10 @@ sub db_array {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -1282,9 +1292,10 @@ sub db_arrayarray {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -1389,9 +1400,10 @@ sub db_hash {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -1480,9 +1492,10 @@ sub db_keyvalhash {
 			} else {
 				# Execute the query
 				try {
-					$sth->execute( @{ $data->{placeholders} } );
+                    $self->bind($sth, $data->{placeholders});
+					$sth->execute();
 				} catch Error with {
-					die $sth->errstr;
+					die (defined($sth->errstr)) ? $sth->errstr : $@;
 				};
 				if (defined($self->{dbh}->errstr)) { die $self->{dbh}->errstr; }
 			}
@@ -1568,6 +1581,17 @@ sub output {
 	return;
 }
 
+# Private methods
+
+sub bind {
+    my ($self, $sth, $p) = @_;
+    
+    for my $i ( 0 .. $#$p ) {
+        local $_ = $p->[$i];
+        $sth->bind_param($i+1 => $_, ((defined && m/^\d+$/) ? SQL_INTEGER : ()));
+    }
+}
+
 1;
 
 __END__
